use bevy::{
    picking::{
        backend::{HitData, PointerHits},
        pointer::{PointerId, PointerLocation},
    },
    prelude::*,
    window::PrimaryWindow,
};
use voronoice::Point;

use super::{CellsEntities, MapMarker, Voronoi};
use crate::camera::CameraMarker;

pub fn picking_backend(
    cam: Query<(&Transform, Entity), With<CameraMarker>>,
    window: Query<&Window, With<PrimaryWindow>>,
    pointers: Query<(&PointerId, &PointerLocation)>,
    map: Query<&Transform, With<MapMarker>>,
    voronoi: Res<Voronoi>,
    cells_entities: Res<CellsEntities>,
    mut output: EventWriter<PointerHits>,
) {
    let (cam, cam_id) = cam.single();
    let window = window.single();
    let map_pos = map.single();
    let mut last_cell = 0;
    for (id, l) in pointers.iter() {
        if let Some(mut pos) = l.location().map(|l| l.position) {
            pos -= window.size() / 2.;
            pos *= cam.scale.xy();
            pos.x += cam.translation.x;
            pos.y -= cam.translation.y;
            if let Some(c) = voronoi
                .0
                .cell(last_cell)
                .iter_path(Point {
                    x: pos.x as f64,
                    y: -pos.y as f64,
                })
                .last()
            {
                last_cell = c;
                output.send(PointerHits {
                    pointer: *id,
                    picks: vec![(
                        cells_entities.0[c],
                        HitData {
                            camera: cam_id,
                            depth: map_pos.translation.z,
                            position: Some(Vec3 {
                                x: pos.x,
                                y: pos.y,
                                z: map_pos.translation.z,
                            }),
                            normal: None,
                        },
                    )],
                    order: map_pos.translation.z,
                });
            }
        }
    }
}
